
dnl --------------------------------------------------------------------------
dnl @synopsis AC_CHECK_TCL([TCL_DIR],[TCL_VERSION],[TCL_VERSION_ALT])
dnl
dnl
AC_DEFUN([AC_CHECK_TCL], [

	TARGET_TCL_DIR=$1
	TCLTK_VERSION=$2
	TCLTK_VERSION_ALT=$3

	FOUND_TCL_INC=no
	FOUND_TCL_LIB=no
	FOUND_TCL=no
	
	TCL_INC=
	TCL_LIB=

	if test -d $TARGET_TCL_DIR ; then
		for i in $TARGET_TCL_DIR/generic $TARGET_TCL_DIR/include/tcl${TCLTK_VERSION} $TARGET_TCL_DIR/include $TARGET_TCL_DIR
		do
			if test -d $i ; then
				AC_CHECK_HEADER($i/tcl.h,
						FOUND_TCL_INC=yes,
						FOUND_TCL_INC=no)

				if test "$FOUND_TCL_INC" = "yes"
				then
					TCL_INC=-I$i
					break
				fi
			fi
		done
	else
		AC_MSG_WARN([Tcl not found in $TARGET_TCL_DIR.])
	fi
	
	if test "$FOUND_TCL_INC" = "yes"
	then
		SAVED_LIBS=$LIBS
		
		# Guess most probable tcl lib location
		for i in $TARGET_TCL_DIR/unix $TARGET_TCL_DIR/lib $TARGET_TCL_DIR
		do
			if test -d $i -a "$FOUND_TCL" = no
			then
				LIBS=$SAVED_LIBS
				
				AC_SEARCH_LIBS(Tcl_Init, tcl${TCLTK_VERSION} tcl${TCLTK_VERSION_ALT}, 
				[
					FOUND_TCL=yes

					# If present, we prefer the static version of the library 
					AC_SEARCH_LIBS(Tcl_Init, $i/libtcl${TCLTK_VERSION}.a $i/libtcl${TCLTK_VERSION_ALT}.a, 
					[
					if test -r $i/libtcl${TCLTK_VERSION}.a ; then
						TCL_LIB="$i/libtcl${TCLTK_VERSION}.a"
					else
						TCL_LIB="$i/libtcl${TCLTK_VERSION_ALT}.a"
					fi
					],
					[TCL_LIB="-L$i -ltcl"], 
					-L$i $SAVED_LIBS -lm)
				],
				[], 
				[-L$i $SAVED_LIBS -lm])
			fi
		done

		LIBS=$SAVED_LIBS
	fi
])

dnl --------------------------------------------------------------------------
dnl @synopsis AC_CHECK_TK([TK_DIR],[TK_VERSION],[TK_VERSION_ALT])
dnl
dnl
AC_DEFUN([AC_CHECK_TK], [

	TARGET_TK_DIR=$1
	TCLTK_VERSION=$2
	TCLTK_VERSION_ALT=$3

	FOUND_TK_INC=no
	FOUND_TK_LIB=no
	FOUND_TK=no
	
	TK_INC=
	TK_LIB=


	# Depending on config expect tk.h to be tk source dir or include path
	# AC_CHECK_HEADER(S) are totally inadequate since it is necessary to
	# include tcl.h, Xlib.h, Xutil.h before tk.h.
	
	if test -d $TARGET_TK_DIR ; then	
		for i in $TARGET_TK_DIR/generic $TARGET_TK_DIR/include/tk${TCLTK_VERSION} $TARGET_TK_DIR/include $TARGET_TK_DIR
		do
			AC_MSG_CHECKING(for tk.h in $i)
			if test -r $i/tk.h ; then
				TK_INC=-I$i
				FOUND_TK_INC=yes
				AC_MSG_RESULT(yes)
				break
			fi
			AC_MSG_RESULT(no)
		done
	else
		AC_MSG_WARN([Tk not found in TARGET_TK_DIR.])
	fi
	
	
	if test "$FOUND_TK_INC" = "yes"
	then
		TK_LIB_DIR=
		
		# Guess most probable tk lib location	
		for i in $TARGET_TK_DIR/unix $TARGET_TK_DIR/lib $TARGET_TK_DIR
		do
			AC_MSG_CHECKING(for the Tk library in $i)			
		        if test -d $i ; then
		                TK_LIB_DIR=$i
		                AC_MSG_RESULT(yes)
		                break
		        fi
		done
		
		if test "$TK_LIB_DIR" != ""
		then
			SAVED_LIBS=$LIBS
			LIBS=""
			
			AC_SEARCH_LIBS(Tk_Init, tk${TCLTK_VERSION} tk${TCLTK_VERSION_ALT},
				[ 
		    	    		FOUND_TK_LIB=yes
				],
		        	AC_MSG_RESULT(no),
				-L${TK_LIB_DIR} ${TCL_LIB} $X_LIBS $X_PRE_LIBS  -lXext -lX11 $X_EXTRA_LIBS ${SAVED_LIBS} -lm)
		
			if test -f ${TK_LIB_DIR}/libtk${TCLTK_VERSION}.a ; then
				TK_LIB="${TK_LIB_DIR}/libtk${TCLTK_VERSION}.a"
			else
				if test -f ${TK_LIB_DIR}/libtk${TCLTK_VERSION_ALT}.a ; then
					TK_LIB="${TK_LIB_DIR}/libtk${TCLTK_VERSION_ALT}.a"
				else 
					TK_LIB="-L${TK_LIB_DIR} $LIBS"
				fi
			fi		
			LIBS=$SAVED_LIBS
			
			if test "$FOUND_TK_LIB" = "yes" ; then
				FOUND_TK=yes
			fi
		fi
	fi
])




dnl --------------------------------------------------------------------------

# AC_CC_OPTION
# -------------------------------
# Check if GCC supports the option provided and add
# it to CFLAGS if available

AC_DEFUN([AC_CC_OPTION],[
	OPTION=$1
	
	ac_save_CFLAGS=$CFLAGS

	AC_MSG_CHECKING([if $CC supports $OPTION])
	
	if test "$GCC" = yes; then
		CFLAGS="$ac_save_CFLAGS $OPTION"
		_AC_COMPILE_IFELSE([AC_LANG_PROGRAM()],
			[AC_MSG_RESULT([yes])],
			[AC_MSG_RESULT([no])
			 CFLAGS=$ac_save_CFLAGS])
	fi

])

# AC_CC_PCH
# -------------------------------
# Check if GCC supports precompiled headers and add it to CFLAGS if available.
# It needs the name of the header file to precompile.

AC_DEFUN([AC_CC_PCH],[

	
	ac_save_CFLAGS=$CFLAGS

	AC_MSG_CHECKING([if $CC supports precompiled headers])
	
	if test "$GCC" = yes; then
		
		CFLAGS="$ac_save_CFLAGS -fpch-preprocess -Winvalid-pch"
		
		_AC_COMPILE_IFELSE([AC_LANG_PROGRAM()],
			[AC_MSG_RESULT([yes])
			 PCH_H=$1
			 PCH_GCH=$PCH_H.gch],
			[AC_MSG_RESULT([no])
			 PCH_H=
			 PCH_GCH=
			 CFLAGS=$ac_save_CFLAGS])
	fi

])
