/*
 * FILE:   controller.h
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.8 $
 * $Date: 2005/05/24 14:44:12 $
 *
 */

#ifndef _CONTROLLER_H
#define _CONTROLLER_H

struct run_env;

/*
 * Error codes
 */
typedef enum {
	ERR_OK = 0,
	ERR_USAGE,
	ERR_UI,
	ERR_DISPLAY,
	ERR_CAPTURE,
	ERR_NET,
	ERR_TRANS,
	ERR_THREADS,
	ERR_CODEC,
} t_ctrlr_err;

/*
 * Subsystems initialization.
 * They return pointers to each subsystem element, so upper levels (ie, UI) can
 * obtain information from them.
 */ 
int			 ctrlr_init_video_codecs (struct run_env *);
struct display		*ctrlr_init_video_display(struct run_env *);
struct vidcap		*ctrlr_init_video_capture(struct run_env *);
struct rtp		*ctrlr_init_network      (struct run_env *);
struct video_tx		*ctrlr_init_transmit     (struct run_env *);

/*
 * Initialization shortcut
 */
t_ctrlr_err       ctrlr_init_all(struct run_env *);

/*
 * Start and stop of the system
 */
t_ctrlr_err       ctrlr_start(struct run_env *);
t_ctrlr_err       ctrlr_done(void);

int               ctrlr_working(void);   /* TRUE if the system is working */

#endif
