/*
 * FILE:   trace.h
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.3 $
 * $Date: 2005/04/18 10:45:37 $
 *
 */

#ifndef _TRACE_H
#define _TRACE_H


struct timeval;

/*
 * Events for trace collection
 */
typedef enum {
	event_frame = 0,
	event_loss,
	event_packet,
	event_fps,
} trace_event_t; 

typedef long trace_value_t;

/* 
 * Traces can be dumped to a file or to a function
 */
typedef enum {
	t_none = 0,
	t_file,
	t_function,
} trace_output_t;

/*
 * Type definition for a tracing function
 */
typedef void (*trace_fun_t)(struct timeval* ts, trace_event_t ev, trace_value_t val);

/*
 * Inits the trace system, witha default trace file/function.
 */
void trace_init(trace_output_t, void*);

/*
 * Registers a trace event with a description and a file or function.
 */
void trace_reg(trace_event_t, char*, void*, trace_output_t );

/*
 * Record an event in a given moment in time.
 */ 
void trace_event( struct timeval*, trace_event_t, trace_value_t);

/*
 * Destroys the trace system
 */
void trace_done(void);

#endif

