/*
 * FILE:   UGController.m
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.3 $
 * $Date: 2005/06/14 15:38:42 $
 *
 */

#include "config.h"
#include "config_unix.h"
#include "debug.h"
#include "trace.h"
#include "environment.h"
#include "controller.h"
#include "video_types.h"
#include "video_capture.h"
#include "video_display.h"
#include "rtp/rtp.h"
#include "rtp/pbuf.h"
#include "video_codec.h"

#import <Cocoa/Cocoa.h>

#import "ui/cocoa/UGRunEnv.h"
#import "ui/cocoa/UGError.h"
#import "ui/cocoa/UGController.h"


/*
 * Tracer function.
 * Will be called with diferent events: some of them should be displayed in
 * the screen, some should be logged in a file...
 */
static void 
tracer(struct timeval* ts, trace_event_t ev, trace_value_t val) {
	UNUSED( ts  );
	UNUSED( ev  );
	UNUSED( val );
}

static UGError	*ugErrorStatic;

static int 
capturer(void *inFD, const char *buffer, int size)
{
        NSAutoreleasePool *pool;
        NSString	  *tmp;
	
	UNUSED(inFD);
	
	pool = [[NSAutoreleasePool alloc] init];
	tmp  = [NSString stringWithCString:buffer length:size];
	
	[ugErrorStatic addText:tmp];
	
	[pool release];
	return size;
}



@implementation UGController

/*******************************************************************************
* Control
******************************************************************************/

- (IBAction)start:(id)sender
{
	NSAutoreleasePool	*autoreleasepool;
	struct run_env		*env;
	
	UNUSED(sender);
	
	env = [ugRunEnv getRunEnv];

	printf("Remote host   : %s\n", env->addr);
	printf("Display device: %s\n", env->display);
	printf("Capture device: %s\n", env->capture);
	printf("Frame rate    : %d\n", env->fps);
	printf("MTU           : %d\n", env->mtu);
        printf("RX port       : %d\n", env->rx_port);
        printf("TX port       : %d\n", env->tx_port);
        printf("TX codec      : %s\n", env->codec);
	
	/*
	 * Capture the scout and stderr output
	 */
	ugErrorStatic  = ugError;
	stdout->_write = capturer;
	stderr->_write = capturer;

	/*
	 * Start each subsystem
	 */
	if (ctrlr_init_video_codecs(env) == FALSE) {	
		[ugError showAsError:self];
		return;
	}
	
	if (ctrlr_init_video_capture(env) == NULL) {	
		[ugError showAsError:self];
		return;
	}
	
	if (ctrlr_init_network(env) == NULL) {
		[ugError showAsError:self];
		return;
	}
	
	if (ctrlr_init_transmit(env) == NULL ) {	
		[ugError showAsError:self];
		return;
	}
	
	if (ctrlr_init_video_display(env) == NULL) {
		[ugError showAsError:self];
		return;
	}
	
	/* Set the tracer function */
	trace_init(t_function, tracer);
	
	autoreleasepool = [[NSAutoreleasePool alloc] init];
	if (ctrlr_start(env) != ERR_OK ) {
		[ugError showAsError:self];
		return;
	}
	[autoreleasepool release];	
}

- (IBAction)stop:(id)sender
{
	UNUSED(sender);
	
	ctrlr_done();	
}

- (BOOL)isWorking
{
	return (ctrlr_working() == TRUE);
}


@end
