/*
 * FILE:   firewire_dv_macosx.c
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *    This product includes software developed by the University of Glasgow.
 * 
 * 4. The name of the University may not be used to endorse or promote 
 *    products derived from this software without specific prior written 
 *    permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.11 $
 * $Date: 2005/06/09 13:33:31 $
 *
 */

#include "config.h"
#include "config_unix.h"

#ifdef HAVE_DARWIN

/*
 * Copyright (c) 1999-2003 WIDE Project
 * All rights reserved.
 *
 * Author : Akimichi OGAWA (akimichi@sfc.wide.ad.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code MUST retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form MUST reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    MUST display the following acknowledgement:
 *      This product includes software developed by Akimichi OGAWA.
 * 4. The name of the author MAY NOT be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "debug.h"
#include "environment.h"
#include "video_types.h"
#include "video_capture.h"
#include "video_capture/firewire_dv_macosx.h"

/* These headers should be moved to config_unix.h maybe... */
#include <Carbon/Carbon.h>
#include <QuickTime/QuickTime.h>
#include <iokit/IOKitLib.h>
#include <DVComponentGlue/IsochronousDataHandler.h>
#include <DVComponentGlue/DeviceControl.h>


#define NUM_FORMATS 2

static OSStatus _macosX_notificationProc __P((IDHGenericEvent *, void *));
static OSStatus _macosX_idh_read __P((IDHGenericEvent *, void *));

/*
 * Mac OS X DV interface
 * 
 * This is an experimental DV implementation. There is no multi-threaded grabbing,
 * and the format is hardcoded for PAL. 
 */

/*
 * DV internal state
 */
struct vidcap_dvmacosx_state {
	unsigned int        fps;		/* FPS */
	display_size_t      format;
	video_colour_mode_t color;	
	char*               read_buffer;	/* Frame read buffer */
	unsigned int        read_buffer_len;
	pthread_rwlock_t    read_lock;
	int                 reading;
	unsigned int        frame_size;
	
	ComponentInstance   theInst;
	QTAtomSpec          videoConfig;
	IDHParameterBlock   isochParamBlock;
};



/*
 * Launch an asynchronous read operation
 */
static int
ieee1394_read(struct vidcap_dvmacosx_state* s)
{
	OSErr				 err;
		
	if (!s->reading) {
		s->reading			  = TRUE;
		s->isochParamBlock.refCon	  = s;
		s->isochParamBlock.buffer         = s->read_buffer;
		s->isochParamBlock.requestedCount = s->frame_size;
		s->isochParamBlock.actualCount    = 0;
		s->isochParamBlock.completionProc = _macosX_idh_read;

		pthread_rwlock_wrlock(&(s->read_lock));
		
		err = IDHRead(s->theInst, &s->isochParamBlock);
		if (err != noErr) {
			fprintf(stderr, "Error: performing IDHRead.\n");
			return -1;
		}
	}
		
	return 1;
}

static OSStatus
_macosX_notificationProc(IDHGenericEvent *event, void *userData)
{
	struct vidcap_dvmacosx_state *s = (struct vidcap_dvmacosx_state*) userData;
	
	debug_msg("notification\n");
	
	IDHNotifyMeWhen(s->theInst,
			event->eventHeader.notificationID, kIDHEventEveryEvent);
	
	return(noErr);
}

/*
 * Finished read notification.
 * This callback function is called when the read finishes.
 */
static OSStatus
_macosX_idh_read(IDHGenericEvent *eventRecord, void *userData)
{
	struct vidcap_dvmacosx_state	*s;
	OSErr				 err = noErr;
	IDHParameterBlock		*pb;
	ComponentInstance		 theInst;
	unsigned int			 len;
	
	
	theInst = userData;
	pb      = (IDHParameterBlock *)eventRecord;
	s	= (struct vidcap_dvmacosx_state*) pb->refCon;
	len     = pb->actualCount;

	assert(s              != NULL);	
	assert(s->read_buffer != NULL);	
	assert(s->frame_size  >= len);

	s->reading = FALSE;

	pthread_rwlock_unlock(&(s->read_lock));		
	
	/* Move the data to the frame buffer */
	if (len <= s->frame_size) {

		err = IDHReleaseBuffer(theInst, pb);
	
		pb->requestedCount = s->frame_size;
		pb->actualCount    = 0;
		pb->completionProc = _macosX_idh_read;
		
//		err = IDHRead(theInst, pb);
//		if (err != noErr) {
//			fprintf(stderr, "Error: performing IDHRead\n");
//			return -1;
//		}
	} else {
		fprintf(stderr, "Error: read length=%d > buffer space=%d.\n",
			len, s->frame_size);
	}
		
	return err;
}


/*
 * Initialize the internal state from the environmental data.
 */
static struct vidcap_dvmacosx_state*
vidcap_dvmacosx_state_init( capture_format_t *format, const char *args ) {
	
	struct vidcap_dvmacosx_state *s = NULL;
	
	UNUSED(args);
	
	s = malloc(sizeof(struct vidcap_dvmacosx_state));
	if (s != NULL) {
		/* We use the first format provided as the format by default */
		s->fps             = format[0].fps;
		s->color           = format[0].colour_mode;
	}
	
	return s;
}

void *
vidcap_dvmacosx_init(capture_format_t *format, const char *args)
{
	struct vidcap_dvmacosx_state	*s = NULL;
	
	u_int32_t			 seed;
	u_int32_t			 num_components;
	ComponentDescription		 desc;
	Component			 aComponent;
	Handle				 aName;
	OSErr				 err;
	ComponentDescription		 aDesc;
	IDHNotificationID		 notificationID;
	IDHDeviceID			 deviceID;
	
	QTAtomContainer			 deviceList = NULL;
	QTAtom				 deviceAtom;
	
	u_int32_t			 cmpFlag;
	u_int32_t			 isoversion;
	
	short				 i, j;
	short				 nDVDevices;
	long				 size;
	
	
	/*
	 * Get avalid state from the environment.
	 */
	s = vidcap_dvmacosx_state_init (format, args);
	
	/* 
	 * Configure the capture devic
	 */
	seed = GetComponentListModSeed();
	
	memset(&desc, 0, sizeof(desc));
	desc.componentType = 'ihlr';
	desc.componentSubType = 0;
	desc.componentFlags = 0;
	desc.componentFlagsMask = 0;
	
	num_components = CountComponents(&desc);
	if (num_components < 1) {
		fprintf(stderr, "Error: No DV device found: %d\n", num_components);
		return NULL;
	}
	
	aComponent = 0;
	aName = NewHandleClear(200);
	
	aComponent = FindNextComponent(aComponent, &desc);
	
	err = GetComponentInfo(aComponent, &aDesc, aName, NULL, NULL);
	if (err) {
		fprintf(stderr, "Error: GetComponentInfo() ERROR %d\n", err);
		return NULL;
	}
	
	s->theInst = OpenDefaultComponent('ihlr', 'dv  ');
	
	err = IDHNewNotification(s->theInst,
				 kIDHDeviceIDEveryDevice,
				 _macosX_notificationProc, s->theInst,
				 &notificationID);
	if (err != noErr) {
		fprintf(stderr, "Error: IDHNewNotification failed\n");
		return NULL;
	}
	
	err = IDHNotifyMeWhen(s->theInst, notificationID, kIDHEventEveryEvent);
	if (err != noErr) {
		fprintf(stderr, "Error: IDHNotifyMeWhen failed\n");
		return NULL;
	}
	
	err = IDHGetDeviceList(s->theInst, &deviceList);
	if (err != noErr) {
		fprintf(stderr, "Error: IDHGetDeviceList failed\n");
		return NULL;
	}
	
	nDVDevices = QTCountChildrenOfType(deviceList,
					   kParentAtomIsContainer,
					   kIDHDeviceAtomType);
	if (nDVDevices < 1) {
		fprintf(stderr,"Error: no DV device found!.\n");
		return NULL;
	}
	
	QTLockContainer(deviceList);
	deviceAtom = QTFindChildByIndex(deviceList,
					kParentAtomIsContainer,
					kIDHUseCMPAtomType, 1, NULL);
	if (deviceAtom == 0) {
		fprintf(stderr, "Error: QTFindChildByIndex failed\n");
		return NULL;
	}
	
	QTCopyAtomDataToPtr(deviceList,
			    deviceAtom, true, sizeof(cmpFlag), &cmpFlag, &size);
	
	deviceAtom = QTFindChildByIndex(deviceList,
					kParentAtomIsContainer,
					kIDHIsochVersionAtomType, 1, NULL);
	if (deviceAtom == 0) {
		return NULL;
	}
	
	QTCopyAtomDataToPtr(deviceList,
			    deviceAtom, true, sizeof(isoversion), &isoversion, &size);
	
	for (i=0; i<nDVDevices; i++) {
		QTAtom		isochAtom, dataAtom;
		u_int32_t	test[2];
		int		nConfigs;
		char		cameraName[256];
		IDHDeviceStatus deviceStatus;
		
		memset(&deviceStatus, 0, sizeof(deviceStatus));
		
		deviceAtom = QTFindChildByIndex(deviceList,
						kParentAtomIsContainer,
						kIDHDeviceAtomType, i + 1, NULL);
		if (deviceAtom == 0) {
			return NULL;
		}
		
		dataAtom = QTFindChildByIndex(deviceList,
					      deviceAtom,
					      kIDHUniqueIDType, 1, NULL);
		if (dataAtom == 0) {
			fprintf(stderr, "Error: performing QTFindChildByIndex\n");			
			return NULL;
		}
		
		QTCopyAtomDataToPtr(deviceList, dataAtom, true, sizeof(test), test, &size);
		
		dataAtom = QTFindChildByIndex(deviceList, deviceAtom,
					      kIDHNameAtomType, 1, NULL);
		if (dataAtom == 0) {
			fprintf(stderr, "Error: performing QTFindChildByIndex\n");			
			return NULL;
		}
		
		QTCopyAtomDataToPtr(deviceList, dataAtom, true, 255, cameraName, &size);
		cameraName[size] = 0;
		
		dataAtom = QTFindChildByIndex(deviceList, deviceAtom,
					      kIDHDeviceIDType, 1, NULL);
		if (dataAtom == 0) {
			fprintf(stderr, "Error: performing QTFindChildByIndex\n");			
			return NULL;
		}
		QTCopyAtomDataToPtr(deviceList, dataAtom,
				    true, sizeof(deviceID),
				    &deviceID, &size);
		
		dataAtom = QTFindChildByIndex(deviceList, deviceAtom, 'ddin', 1, NULL);
		if (dataAtom == 0) {
			fprintf(stderr, "Error: performing QTFindChildByIndex\n");			
			return NULL;
		}
		
		QTCopyAtomDataToPtr(deviceList, dataAtom,
				    true, sizeof(deviceStatus), &deviceStatus, &size);
		
		if (deviceStatus.inputStandard == ntscIn) {
			printf("NTSC device found!.\n");		
			s->frame_size = 120000;
		}
		else if (deviceStatus.inputStandard == palIn) {
			printf("PAL device found!.\n");
			s->frame_size = 144000;
		}
		else {
			fprintf(stderr, "Error: Unknown DV standard : %d\n", deviceStatus.inputStandard);
			return NULL;
		}

		isochAtom = QTFindChildByIndex(deviceList, deviceAtom,
					       kIDHIsochServiceAtomType, 1, NULL);
		if (isochAtom == 0) {
			fprintf(stderr, "Error: performing QTFindChildByIndex\n");			
			return NULL;
		}
		
		nConfigs = QTCountChildrenOfType(deviceList, isochAtom,
						 kIDHIsochModeAtomType);
		s->videoConfig.atom = 0;
		
		for (j=0; j<nConfigs; ++j) {
			OSType	mediaType;
			QTAtom	configAtom, mediaAtom;
			
			configAtom = QTFindChildByIndex(deviceList, isochAtom,
							kIDHIsochModeAtomType, j+1, NULL);
			if (configAtom == 0) {
				fprintf(stderr, "Error: performing QTFindChildByIndex\n");
				return NULL;
			}
			
			mediaAtom = QTFindChildByIndex(deviceList, configAtom,
						       kIDHIsochMediaType, 1, NULL);
			if (mediaAtom == 0) {
				fprintf(stderr, "Error: performing QTFindChildByIndex\n");
				return NULL;
			}
			
			QTCopyAtomDataToPtr(deviceList, mediaAtom, true, sizeof(mediaType),
					    &mediaType, &size);
			if (mediaType == kIDHVideoMediaAtomType) {
				s->videoConfig.container = deviceList;
				s->videoConfig.atom      = configAtom;
			}
		}
	}
	
	if (s->videoConfig.atom == 0) {
		free(s);
		return NULL;
	}
	
	err = IDHSetDeviceConfiguration(s->theInst, &s->videoConfig);
	if (err != noErr) {
		free(s);		
		return NULL;
	}
	
	err = IDHOpenDevice(s->theInst, kIDHOpenForReadTransactions);
	if (err != noErr) {
		free(s);
		return NULL;
	}
	
	/*
	 * Parameters for the asynchonous read
	 */
	s->isochParamBlock.buffer         = NULL;
	s->isochParamBlock.requestedCount = s->frame_size;
	s->isochParamBlock.actualCount    = 0;
	s->isochParamBlock.refCon         = (void *)s->theInst;
	s->isochParamBlock.completionProc = _macosX_idh_read;
	
	/* Allocate memory for reads */
	debug_msg("Allocating %dbytes for grabbing.\n", s->frame_size);
	s->read_buffer = malloc(s->frame_size);
	if(s == NULL) {
		fprintf(stderr, "Error: allocating memory for the grabber.\n");
		free(s);
		return NULL;
	}
	
	s->reading = FALSE;
	
	/* Create the read mutex... */
	if( pthread_rwlock_init(&(s->read_lock), NULL) != 0) {
		fprintf(stderr, "Error creating the read lock.\n");
		free(s);
		return NULL;
	}
	
	s->isochParamBlock.buffer         = s->read_buffer;	
	
	ieee1394_read(s);
	
	return (void*) s;
}

void
vidcap_dvmacosx_done(void *state)
{
	struct vidcap_dvmacosx_state *s = (struct vidcap_dvmacosx_state *) state;
	
	assert(s != NULL);
	free(s);
}

struct video_frame *
vidcap_dvmacosx_grab(void *state)
{
	struct vidcap_dvmacosx_state	*s = (struct vidcap_dvmacosx_state *) state;
	struct video_frame		*vf;
		
	assert(s != NULL);
	
	vf = (struct video_frame *) malloc(sizeof(struct video_frame));
	if (vf != NULL) {				
		if (pthread_rwlock_rdlock(&(s->read_lock)) != 0) {
			fprintf(stderr, "Error: waiting for read lock.\n");
			return NULL;
		}
		
		/* Now, there should be a frame in s->read_buffer */
		vf->colour_mode	= s->color;
		vf->width	= display_get_width(s->format);
		vf->height	= display_get_height(s->format);
		vf->data	= (unsigned char*)s->read_buffer;		
		vf->data_len	= s->frame_size;
		
		if (pthread_rwlock_unlock(&(s->read_lock)) != 0) {
			fprintf(stderr, "Error: unlocking the read lock.\n");
			return NULL;
		}		
		
		/* Launch next read */
		ieee1394_read(s);
		
        } else {
                fprintf(stderr,"Error: not enough space to allocate a video frame.\n");
	}
	
	return vf;
}

struct vidcap_type *
vidcap_dvmacosx_probe(const struct run_env* environ)
{
        struct vidcap_type		*vt	 = NULL;
	struct vidcap_dvmacosx_state	*s	 = NULL;
	capture_format_t		*cformat = NULL;
	u_int32_t			 seed;
	u_int32_t			 num_components;
	ComponentDescription		 desc;
	
        assert( environ != NULL);

	seed = GetComponentListModSeed();
	
	memset(&desc, 0, sizeof(desc));
	
	desc.componentType        = 'ihlr';
	desc.componentSubType     = 0;
	desc.componentFlags       = 0;
	desc.componentFlagsMask   = 0;
	
	num_components = CountComponents(&desc);
	
	if (num_components > 0) {

		/*
		 * TODO: we could probably ask the number of capture formats and
		 * their properties in Mac...
		 */
		cformat = malloc(NUM_FORMATS * sizeof(capture_format_t));
		if(cformat != NULL) {
			cformat[0].size        = DS_PAL;
			cformat[0].colour_mode = YUV_422;
			cformat[0].fps         = 25;
			cformat[0].encoding    = dv;
			
			cformat[1].size        = DS_NTSC;
			cformat[1].colour_mode = YUV_422;
			cformat[1].fps         = 25;
			cformat[1].encoding    = dv;	
		} else {
			return NULL;
		}		
		
		/*
		 * Start the environment and open the device
		 */
		s = vidcap_dvmacosx_state_init( cformat, environ->capture_args );

		vt = (struct vidcap_type *) malloc(sizeof(struct vidcap_type));
		if (vt != NULL) {
			vt->id          = VIDCAP_DVMACOSX_ID;
			vt->name        = "dv";
			vt->description = "IEEE-1394/DV";
			vt->props       = 0;
			vt->formats	= cformat;
			vt->num_formats = NUM_FORMATS;
		}
		
		vidcap_dvmacosx_done(s);		
	}
	
        return vt;
}


#endif
