/*
 * FILE:   display.h
 * AUTHOR: Colin Perkins <csp@csperkins.org>
 *
 * Copyright (c) 2001-2003 University of Southern California
 * Copyright (c)      2004 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *      This product includes software developed by the University of Southern
 *      California Information Sciences Institute.
 * 
 * 4. Neither the name of the University nor of the Institute may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.29 $
 * $Date: 2005/07/14 19:32:36 $
 *
 */

#ifndef _VIDEO_DISPLAY_H
#define _VIDEO_DISPLAY_H

/*
 * Interface to probing the valid display types. 
 */
typedef uint32_t	display_id_t;

/*
 * Display properties
 */
typedef uint32_t	display_prop_t;

#define DISPLAY_NEEDS_PARAMS	0x0001	/* Needs initialization params */
#define DISPLAY_PARAM_FILENAME	0x0010	/* Needs a filename */
#define DISPLAY_PARAM_FPS	0x0020  /* Needs the FPS */
#define DISPLAY_PARAM_SSIZE	0x0040  /* Needs the screen size */

typedef struct {
	display_size_t		size;
	video_colour_mode_t	colour_mode;
	int			num_images;	/* Maximum displayable images, -1 = unlimited */
} display_format_t;

typedef struct {
	display_id_t		 id;
	const char		*name;		/* Single word name 		*/
	const char		*description;
	display_prop_t		 props;
	display_format_t	*formats;	/* Array of supported formats 	*/
	unsigned int		 num_formats;	/* Size of the array 		*/
} display_type_t;

/*
 * Return codes from the display
 */
typedef enum {
	DISPLAY_OK,
	DISPLAY_EXIT_REQ,
	DISPLAY_ERR,
	DISPLAY_RETRY,
	DISPLAY_IGN,
} display_result_t;

struct run_env;	 /* Complete definition is in environment.h, but not needed here [csp] */

int		 display_init_devices(const struct run_env* environ, const char* display_args);
void		 display_free_devices(void);
unsigned int	 display_get_device_count(void);
display_type_t	*display_get_device_details(unsigned int index);

/* 
 * Interface to initialize displays, and playout video.
 */

struct display;

struct display		*display_init(display_id_t id, display_format_t *format, const char* display_args);
void 			 display_done(struct display *d);
struct video_frame	*display_get_frame(struct display *d, display_size_t size, video_colour_mode_t color);
display_result_t 	 display_put_frame(struct display *d, struct video_frame *frame);
video_colour_mode_t	 display_get_colour_mode(struct display *d);

#endif
