/*
 * FILE:   video_types.c
 * AUTHOR: Alvaro Saurin <saurin@dcs.gla.ac.uk>
 *
 * Copyright (c) 2005 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *      This product includes software developed by the University of Southern
 *      California Information Sciences Institute.
 * 
 * 4. Neither the name of the University nor of the Institute may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.1 $
 * $Date: 2005/06/03 16:07:10 $
 *
 */

#include "config.h"
#include "config_unix.h"
#include "debug.h"
#include "video_types.h"

typedef struct {
	display_size_t		 id;
	char			*description;
	int			 width;
	int			 height;
} display_size_ext_t;

static display_size_ext_t displays_table[] = {
	{ DS_NONE,   "(none)",                     0,    0 },
	{ DS_QCIF,   "Quarter CIF",              176,  144 },
	{ DS_CIF,    "CIF",                      352,  288 },
	{ DS_SCIF,   "Super CIF",                702,  576 },
	{ DS_NTSC,   "NTSC",                     720,  480 },
	{ DS_PAL,    "PAL",                      720,  576 },
	{ DS_720p,   "SMPTE 296M - HDTV 720p",  1280,  720 },
	{ DS_1080i,  "SMPTE 274M - HDTV 1080i", 1920, 1080 },
};

#define NUM_DISPLAYS (sizeof(displays_table)/sizeof(display_size_ext_t))

unsigned int	
display_get_width(display_size_t d)
{
	assert(d < NUM_DISPLAYS);
	return displays_table[d].width;
}

unsigned int	
display_get_height(display_size_t d)
{
	assert(d < NUM_DISPLAYS);
	return displays_table[d].height;
}


