/*
 * FILE:   video_types.h
 * AUTHOR: Colin Perkins <csp@csperkins.org
 *
 * Copyright (c) 2005 University of Glasgow
 * Copyright (c) 2002 University of Southern California
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *      This product includes software developed by the University of Southern
 *      California Information Sciences Institute.
 * 
 * 4. Neither the name of the University nor of the Institute may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.10 $
 * $Date: 2005/06/03 16:08:54 $
 *
 */

#ifndef _VIDEO_TYPES_H
#define _VIDEO_TYPES_H


typedef enum {
	NONE,
	RGB,
	YUV_422,
	YUV_422_DVS10,
} video_colour_mode_t;

/*
 * Common image sizes.
 */
typedef enum {
	DS_NONE,
	DS_QCIF,	/* Quarter CIF                     */
	DS_CIF,		/* CIF - Common Interchange Format */
	DS_SCIF,	/* Super CIF                       */
	DS_NTSC,	/* NTSC - North American TV        */
	DS_PAL,		/* PAL - European TV               */
	DS_720p,	/* SMPTE 296M - HDTV 720p          */
	DS_1080i,	/* SMPTE 274M - HDTV 1080i         */
} display_size_t;

unsigned int	display_get_width(display_size_t d);
unsigned int	display_get_height(display_size_t d);


/* 
 * Would it make sense to convert this to:
 *
 *	struct video_frame {
 *		video_colour_mode_t	 colour_mode;
 *		unsigned		 total_width;
 *		unsigned		 total_height;
 *		unsigned		 active_width;
 *		unsigned		 active_height;
 *		unsigned		 x_offset;
 *		unsigned		 y_offset;
 *		unsigned		 depth;
 *		unsigned char		*data;
 *		unsigned		 data_len;
 *	};
 *
 * with the codecs writing into a rectange offset by (x_offset, y_offset)
 * of size (active_width, active_height) within the real frame?
 *
 */

struct video_frame {
	video_colour_mode_t	 colour_mode;
	unsigned		 width;
	unsigned		 height;
	unsigned char		*data;
	unsigned		 data_len;	/* Number of octets of data present */
};

#endif

